#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Specific exceptions for PyMetaWear

Created by hbldh <henrik.blidh@nedomkull.com> on 2016-03-30

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import


class PyMetaWearException(Exception):
    """Main exception."""
    pass


class PyMetaWearConnectionTimeout(PyMetaWearException):
    """Connection could not be established in time."""
    pass


class PyMetaWearDownloadTimeout(PyMetaWearException):
    """Raised when connection is lost during download of logging data."""
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Gyroscope module
----------------

Created by hbldh <henrik.blidh@nedomkull.com> on 2016-04-26

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import re
import logging

from pymetawear import libmetawear
from pymetawear.exceptions import PyMetaWearException
from mbientlab.metawear.cbindings import GyroBmi160Odr, GyroBmi160Range
from pymetawear.modules.base import PyMetaWearLoggingModule, Modules, data_handler

log = logging.getLogger(__name__)


def require_bmi160(f):
    def wrapper(*args, **kwargs):
        if getattr(args[0], 'gyro_r_class', None) is None:
            raise PyMetaWearException("There is not Gyroscope "
                                      "module of your MetaWear board!")
        return f(*args, **kwargs)

    return wrapper


class GyroscopeModule(PyMetaWearLoggingModule):
    """MetaWear gyroscope module implementation.

    :param ctypes.c_long board: The MetaWear board pointer value.
    :param int module_id: The module id of this gyroscope
        component, obtained from ``libmetawear``.
    :param bool debug: If ``True``, module prints out debug information.

    """

    def __init__(self, board, module_id):
        super(GyroscopeModule, self).__init__(board)
        self.module_id = module_id

        self.high_frequency_stream = False

        self.odr = {}
        self.fsr = {}

        self.current_odr = 0
        self.current_fsr = 0

        if self.module_id == Modules.MBL_MW_MODULE_NA:
            # No gyroscope present!
            self.gyro_r_class = None
            self.gyro_o_class = None
        else:
            self.gyro_r_class = GyroBmi160Range
            self.gyro_o_class = GyroBmi160Odr

        if self.gyro_o_class is not None:
            # Parse possible output data rates for this gyroscope.
            for key, value in vars(self.gyro_o_class).items():
                if re.search('_([0-9]+)Hz', key) and key is not None:
                    self.odr.update({key[1:-2]: value})

        if self.gyro_r_class is not None:
            # Parse possible ranges for this gyroscope.
            for key, value in vars(self.gyro_r_class).items():
                if re.search('_([0-9]+)dps', key) and key is not None:
                    self.fsr.update({key[1:-3]: value})

    def __str__(self):
        return "{0} {1}: Data rates (Hz): {2}, Data ranges (dps): {3}".format(
            self.module_name, self.sensor_name,
            [float(k) for k in sorted(self.odr.keys(),
                                      key=lambda x: (float(x)))],
            [k for k in sorted(self.fsr.keys())])

    def __repr__(self):
        return str(self)

    @property
    def module_name(self):
        return "Gyroscope"

    @property
    def sensor_name(self):
        if self.gyro_r_class is not None:
            return self.gyro_r_class.__name__.replace(
                'Gyro', '').replace('Range', '')
        else:
            return ''

    @property
    @require_bmi160
    def data_signal(self):
        if self.high_frequency_stream:
            return libmetawear.mbl_mw_gyro_bmi160_get_high_freq_rotation_data_signal(
                self.board)
        else:
            return libmetawear.mbl_mw_gyro_bmi160_get_rotation_data_signal(
                self.board)

    def _get_odr(self, value):
        sorted_ord_keys = sorted(self.odr.keys(), key=lambda x: (float(x)))
        diffs = [abs(value - float(k)) for k in sorted_ord_keys]
        min_diffs = min(diffs)
        if min_diffs > 0.5:
            raise ValueError(
                "Requested ODR ({0}) was not part of possible values: {1}".format(
                    value, [float(x) for x in sorted_ord_keys]))
        k = sorted_ord_keys[diffs.index(min_diffs)]
        return self.odr.get(k)

    def _get_fsr(self, value):
        sorted_ord_keys = sorted(self.fsr.keys(), key=lambda x: (float(x)))
        diffs = [abs(value - float(k)) for k in sorted_ord_keys]
        min_diffs = min(diffs)
        if min_diffs > 0.1:
            raise ValueError(
                "Requested FSR ({0}) was not part of possible values: {1}".format(
                    value, [float(x) for x in sorted(self.fsr.keys())]))
        k = sorted_ord_keys[diffs.index(min_diffs)]
        return self.fsr.get(k)

    @require_bmi160
    def get_current_settings(self):
        return "data_rate in Hz: {} data_range in deg/sec: {}".format(
            self.current_odr, self.current_fsr)

    @require_bmi160
    def get_possible_settings(self):
        return {
            'data_rate': [float(x) for x in sorted(
                self.odr.keys(), key=lambda x: (float(x)))],
            'data_range': [x for x in sorted(self.fsr.keys())]
        }

    @require_bmi160
    def set_settings(self, data_rate=None, data_range=None):
        """Set gyroscope settings.

         Can be called with two or only one setting:

         .. code-block:: python

            mwclient.gyroscope.set_settings(data_rate=200.0, data_range=8.0)

        will give the same result as

        .. code-block:: python

            mwclient.gyroscope.set_settings(data_rate=200.0)
            mwclient.gyroscope.set_settings(data_range=1000.0)

        albeit that the latter example makes two writes to the board.

        Call :meth:`~get_possible_settings` to see which values
        that can be set for this sensor.

        :param float data_rate: The frequency of gyroscope updates in Hz.
        :param float data_range: The measurement range in the unit ``dps``,
            degrees per second.

        """
        if data_rate is not None:
            odr = self._get_odr(data_rate)
            log.debug("Setting Gyroscope ODR to {0}".format(odr))
            libmetawear.mbl_mw_gyro_bmi160_set_odr(self.board, odr)
            self.current_odr = data_rate
        if data_range is not None:
            fsr = self._get_fsr(data_range)
            log.debug("Setting Gyroscope FSR to {0}".format(fsr))
            libmetawear.mbl_mw_gyro_bmi160_set_range(self.board, fsr)
            self.current_fsr = data_range
        if (data_rate is not None) or (data_range is not None):
            libmetawear.mbl_mw_gyro_bmi160_write_config(self.board)

    @require_bmi160
    def notifications(self, callback=None):
        """Subscribe or unsubscribe to gyroscope notifications.

        Convenience method for handling gyroscope usage.

        Example:

        .. code-block:: python

            def handle_notification(data):
                # Handle dictionary with [epoch, value] keys.
                epoch = data["epoch"]
                xyz = data["value"]
                print(str(data))

            mwclient.gyroscope.notifications(handle_notification)

        :param callable callback: Gyroscope notification callback function.
            If `None`, unsubscription to gyroscope notifications is registered.

        """
        if callback is None:
            self.stop()
            self.toggle_sampling(False)
            super(GyroscopeModule, self).notifications(None)
        else:
            super(GyroscopeModule, self).notifications(data_handler(callback))
            self.toggle_sampling(True)
            self.start()

    @require_bmi160
    def start(self):
        """Switches the gyroscope to active mode."""
        libmetawear.mbl_mw_gyro_bmi160_start(self.board)

    @require_bmi160
    def stop(self):
        """Switches the gyroscope to standby mode."""
        libmetawear.mbl_mw_gyro_bmi160_stop(self.board)

    @require_bmi160
    def toggle_sampling(self, enabled=True):
        """Enables or disables gyroscope sampling.

        :param bool enabled: Desired state of the gyroscope.

        """
        if enabled:
            libmetawear.mbl_mw_gyro_bmi160_enable_rotation_sampling(self.board)
        else:
            libmetawear.mbl_mw_gyro_bmi160_disable_rotation_sampling(self.board)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Settings module
---------------

Created by hbldh <henrik.blidh@nedomkull.com> on 2016-04-14

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import logging
import warnings
from ctypes import c_ubyte, create_string_buffer, cast, POINTER

from pymetawear import libmetawear
from pymetawear.modules.base import PyMetaWearModule, data_handler

log = logging.getLogger(__name__)


class SettingsModule(PyMetaWearModule):
    """MetaWear Settings module implementation.

    :param ctypes.c_long board: The MetaWear board pointer value.
    :param bool debug: If ``True``, module prints out debug information.

    """

    def __init__(self, board):
        super(SettingsModule, self).__init__(board)

        self.current_name = None
        self.current_tx_power = None
        self.current_ad_interval = None
        self.current_ad_timeout = None
        self.current_min_conn_interval = None
        self.current_max_conn_interval = None
        self.current_latency = None
        self.current_timeout = None

        self.possible_tx_power = [4, 0, -4, -8, -12, -16, -20, -30]

    def __str__(self):
        return "{0}: {1}".format(self.module_name, self.sensor_name)

    def __repr__(self):
        return super(SettingsModule, self).__repr__()

    @property
    def module_name(self):
        return 'Settings'

    @property
    def sensor_name(self):
        return self.module_name

    @property
    def data_signal(self):
        return libmetawear.mbl_mw_settings_get_battery_state_data_signal(self.board)

    def set_device_name(self, name=None):
        if name is not None and len(name) <= 8:
            self.current_name = name
            log.debug("Setting device name to {0}".format(name))
            fname = create_string_buffer(name, 8)
            bname = cast(fname, POINTER(c_ubyte))
            libmetawear.mbl_mw_settings_set_device_name(self.board, bname, len(fname.raw))  
        elif self._debug:
            log.debug("{0} is not a valid name".format(name))
    
    def set_tx_power(self, power=None):
        if power is not None:
            if power in self.possible_tx_power:
                self.current_tx_power = power
                log.debug("Setting device TX power to {0}".format(power))
                libmetawear.mbl_mw_settings_set_tx_power(self.board, power) 
            elif self._debug:
                log.debug("{0} is not a valid value for TX power. Valid values are {1}".format(power, self.possible_tx_power))

    def set_ad_interval(self, interval=None, timeout=None):
        if interval is not None and timeout is not None:
            if (interval >= 0 and interval <= 65535) and (timeout >= 0 and timeout <= 180):
                self.current_ad_interval = interval
                self.current_ad_timeout = timeout
                log.debug("Setting device interval to {0} and timeout to {1}".format(interval, timeout))
                libmetawear.mbl_mw_settings_set_ad_interval(self.board, interval, timeout) 
            elif self._debug:
                log.debug("Invalid values for timeout and interval")
   
    def start_advertising(self):
        log.debug("Setting device to start advertising")
        libmetawear.mbl_mw_settings_start_advertising(self.board) 

    def set_connection_parameters(self, min_conn_interval=None, max_conn_interval=None, latency=None, timeout=None):
        if min_conn_interval is not None and max_conn_interval is not None and latency is not None and timeout is not None:
            if ((min_conn_interval >= 7.5) and (max_conn_interval <= 4000) and (latency >= 0 and latency <= 1000) and (timeout >= 10 and timeout <= 32000)):
                self.current_max_conn_interval = max_conn_interval
                self.current_max_conn_interval = max_conn_interval
                self.current_latency = latency 
                self.current_timeout = timeout
                log.debug("Setting device min connection interval to {0}, max connection interval to {1}, latency to {2}, and timeout to {3}".format(min_conn_interval, max_conn_interval, latency, timeout))
                libmetawear.mbl_mw_settings_set_connection_parameters(self.board, min_conn_interval, max_conn_interval, latency, timeout) 
            elif self._debug:
                log.debug("Invalid values for min connection interval, max connection interval, timeout and latency")
    
    def set_scan_response(self, response=None):
        if response is not None:
            self.current_scan_response = response
            log.debug("Setting scan response to {0}".format(response))
            fresponse = create_string_buffer(response)
            bresponse = cast(fresponse, POINTER(c_ubyte))
            libmetawear.mbl_mw_settings_set_device_name(self.board, bresponse, len(fresponse))  
    
    def notifications(self, callback=None):
        """Subscribe or unsubscribe to battery notifications.

        Convenience method for handling battery notifications.

        The data to the callback method comes as a tuple of two integer
        values, the first one representing the voltage and the second one
        is an integer in [0, 100] representing battery percentage.

        Example:

        .. code-block:: python

            def battery_callback(data):
                epoch = data[0]
                battery = data[1]
                print("[{0}] Voltage: {1}, Charge: {2}".format(
                    epoch, battery[0], battery[1]))

            mwclient.settings.notifications(battery_callback)
            mwclient.settings.read_battery_state()

        :param callable callback: Battery data notification callback
            function. If `None`, unsubscription to battery notifications
            is registered.

        """
        super(SettingsModule, self).notifications(
            data_handler(callback) if callback is not None else None)

    def read_battery_state(self):
        """Triggers a battery state notification.

        N.B. that a :meth:`~notifications` call that registers a
        callback for battery state should have been done prior to calling
        this method.

        """
        if self.callback is None:
            warnings.warn("No battery callback is registered!", RuntimeWarning)
        libmetawear.mbl_mw_datasignal_read(self.data_signal)
    
    def disconnected_event(self):
        """Create a disconnect event.

        Convenience method for sending commands during a disconnect event.

        """
        event = libmetawear.mbl_mw_settings_get_disconnect_event(self.board)
        return event



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:mod:`accelerometer`
==================

Updated by lkasso <hello@mbientlab.com>
Created by hbldh <henrik.blidh@nedomkull.com>
Created on 2016-04-10

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import time

from pymetawear.discover import select_device
from pymetawear.client import MetaWearClient

address = select_device()
c = MetaWearClient(str(address), debug=True)
print("New client created: {0}".format(c))

print("Get possible barometer settings...")
settings = c.barometer.get_possible_settings()
print(settings)

time.sleep(1.0)

print("Write barometer settings...")
c.barometer.set_settings(oversampling='HIGH',
                         iir_filter='AVG_8',
                         standby_time=250.0)

time.sleep(1.0)

print("Get current barometer settings...")
settings = c.barometer.get_current_settings()
print(settings)

time.sleep(5.0)

print("Subscribing to barometer signal notifications...")
c.barometer.notifications(lambda data: print(data))

time.sleep(20.0)

print("Unsubscribe to notification...")
c.barometer.notifications(None)

time.sleep(5.0)

c.disconnect()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Accelerometer module
--------------------

Created by hbldh <henrik.blidh@nedomkull.com> on 2016-04-14
Modified by lkasso <hello@mbientlab.com>

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import re
import logging
from ctypes import c_float

from pymetawear import libmetawear
from mbientlab.metawear.cbindings import AccBma255Odr, AccBmi160Odr, \
    AccBmi160StepCounterMode, AccBoschOrientationMode, AccBoschRange, \
    AccMma8452qOdr, AccMma8452qRange, Const
from pymetawear.modules.base import PyMetaWearLoggingModule, data_handler

log = logging.getLogger(__name__)

_settings_map = {
    Const.MODULE_ACC_TYPE_BMA255: (AccBma255Odr, AccBoschRange),
    Const.MODULE_ACC_TYPE_BMI160: (AccBmi160Odr, AccBoschRange),
    Const.MODULE_ACC_TYPE_MMA8452Q: (AccMma8452qOdr, AccMma8452qRange)
}


class AccelerometerModule(PyMetaWearLoggingModule):
    """MetaWear accelerometer module implementation.

    :param ctypes.c_long board: The MetaWear board pointer value.
    :param int module_id: The module id of this accelerometer
        component, obtained from ``libmetawear``.
    :param bool debug: If ``True``, module prints out debug information.

    """

    def __init__(self, board, module_id):
        super(AccelerometerModule, self).__init__(board)
        self.module_id = module_id

        self.high_frequency_stream = False

        self.current_odr = 0
        self.current_fsr = 0

        self.odr = {}
        self.fsr = {}

        acc_odr_class, acc_fsr_class = _settings_map.get(module_id)

        if acc_odr_class is not None:
            # Parse possible output data rates for this accelerometer.
            for key, value in vars(acc_odr_class).items():
                if re.search('^_([0-9]+)_*([0-9]*)Hz', key) and key is not None:
                    self.odr.update({key[1:-2].replace("_","."): value})

        if acc_fsr_class is not None:
            # Parse possible output data ranges for this accelerometer.
            for key, value in vars(acc_fsr_class).items():
                if re.search('^_([0-9]+)G', key) and key is not None:
                    self.fsr.update({key[1:-1]: value})

        self._acc_sensor_name = acc_odr_class.__name__.replace(
            'Acc', '').replace('Odr', '')

    def __str__(self):
        return "{0} {1}: Data rates (Hz): {2}, Data ranges (g): {3}".format(
            self.module_name, self.sensor_name,
            [float(k) for k in sorted(self.odr.keys(),
                                      key=lambda x:(float(x)))],
            [k for k in sorted(self.fsr.keys())])

    def __repr__(self):
        return str(self)

    @property
    def module_name(self):
        return "Accelerometer"

    @property
    def sensor_name(self):
        return self._acc_sensor_name

    @property
    def data_signal(self):
        if self.high_frequency_stream:
            return libmetawear.mbl_mw_acc_get_high_freq_acceleration_data_signal(self.board)
        else:
            return libmetawear.mbl_mw_acc_get_acceleration_data_signal(self.board)

    def _get_odr(self, value):
        sorted_ord_keys = sorted(self.odr.keys(), key=lambda x: (float(x)))
        diffs = [abs(value - float(k)) for k in sorted_ord_keys]
        min_diffs = min(diffs)
        if min_diffs > 0.5:
            raise ValueError(
                "Requested ODR ({0}) was not part of possible values: {1}".format(
                    value, [float(x) for x in sorted_ord_keys]))
        return float(value)

    def _get_fsr(self, value):
        sorted_ord_keys = sorted(self.fsr.keys(), key=lambda x: (float(x)))
        diffs = [abs(value - float(k)) for k in sorted_ord_keys]
        min_diffs = min(diffs)
        if min_diffs > 0.1:
            raise ValueError(
                "Requested FSR ({0}) was not part of possible values: {1}".format(
                    value, [x for x in sorted(self.fsr.keys())]))
        return float(value)

    def get_current_settings(self):
        return "data_rate in Hz: {} data_range in Gs: {}".format(
            self.current_odr, self.current_fsr)

    def get_possible_settings(self):
        return {
            'data_rate in Hz': [float(x) for x in sorted(
                self.odr.keys(), key=lambda x:(float(x)))],
            'data_range in Gs': [int(x) for x in sorted(self.fsr.keys())]
        }

    def set_settings(self, data_rate=None, data_range=None):
        """Set accelerometer settings.

         Can be called with two or only one setting:

         .. code-block:: python

            mwclient.set_accelerometer_settings(data_rate=200.0, data_range=8.0)

        will give the same result as

        .. code-block:: python

            mwclient.set_accelerometer_settings(data_rate=200.0)
            mwclient.set_accelerometer_settings(data_range=8.0)

        albeit that the latter example makes two writes to the board.

        Call :meth:`~get_possible_settings` to see which values
        that can be set for this sensor.

        :param float data_rate: The frequency of accelerometer updates in Hz.
        :param float data_range: The measurement range in the unit ``g``.

        """

        if data_rate is not None:
            self.current_odr = data_rate
            odr = self._get_odr(data_rate)
            log.debug("Setting Accelerometer ODR to {0}".format(odr))
            libmetawear.mbl_mw_acc_set_odr(self.board, c_float(odr))
       
        if data_range is not None:
            self.current_fsr = data_range
            fsr = self._get_fsr(data_range)
            log.debug("Setting Accelerometer FSR to {0}".format(fsr))
            libmetawear.mbl_mw_acc_set_range(self.board, c_float(fsr))

        if (data_rate is not None) or (data_range is not None):
            self.current_odr = data_rate
            self.current_fsr = data_range
            libmetawear.mbl_mw_acc_write_acceleration_config(self.board)

    def notifications(self, callback=None):
        """Subscribe or unsubscribe to accelerometer notifications.

        Convenience method for handling accelerometer usage.

        Example:

        .. code-block:: python

            def handle_acc_notification(data)
                # Handle dictionary with [epoch, value] keys.
                epoch = data["epoch"]
                xyz = data["value"]
                print(str(data))

            mwclient.accelerometer.notifications(handle_acc_notification)

        :param callable callback: Accelerometer notification callback function.
            If `None`, unsubscription to accelerometer notifications
            is registered.

        """

        if callback is None:
            self.stop()
            self.toggle_sampling(False)
            super(AccelerometerModule, self).notifications(None)
        else:
            super(AccelerometerModule, self).notifications(
                data_handler(callback))
            self.toggle_sampling(True)
            self.start()

    def start(self):
        """Switches the accelerometer to active mode."""
        libmetawear.mbl_mw_acc_start(self.board)
        log.debug("Start Sampling (Accelerometer)")

    def stop(self):
        """Switches the accelerometer to standby mode."""
        libmetawear.mbl_mw_acc_stop(self.board)

    def toggle_sampling(self, enabled=True):
        """Enables or disables accelerometer sampling.

        :param bool enabled: Desired state of the accelerometer.

        """
        if enabled:
            libmetawear.mbl_mw_acc_enable_acceleration_sampling(self.board)
        else:
            libmetawear.mbl_mw_acc_disable_acceleration_sampling(self.board)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Main module for PyMetaWear

.. moduleauthor:: hbldh <henrik.blidh@nedomkull.com>

Created on 2016-03-30

"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import logging

from mbientlab.metawear import MetaWear, libmetawear

from pymetawear import add_stream_logger, modules


log = logging.getLogger(__name__)

_model_names = [
    "Unknown",
    "MetaWear R",
    "MetaWear RG",
    "MetaWear RPro",
    "MetaWear C",
    "MetaWear CPro",
    "MetaEnvironment",
    "MetaDetector",
    "MetaHealth",
    "MetaTracker",
    "MetaMotion R",
    "MetaMotion C"
]


class MetaWearClient(object):
    """A MetaWear communication client.

    This client bridges the gap between the
    `MetaWear C++ API <https://github.com/mbientlab/Metawear-CppAPI>`_
    and a GATT communication package in Python. It provides Pythonic
    interface to using the MetaWear boards, allowing for rapid
    development and testing.

    :param str address: A Bluetooth MAC address to a MetaWear board.
    :param str device: Specifying which Bluetooth device to use. Defaults
        to ``hci0`` on Linux. Not available on Windows.
    :param bool connect: If client should connect automatically, or wait for
        explicit :py:meth:`~MetaWearClient.connect` call. Default is ``True``.
    :param bool debug: If printout of all sent and received
        data should be done.

    """

    def __init__(self, address, device='hci0', connect=True, debug=False):
        """Constructor."""
        self._address = address
        self._debug = debug
        self._connect = connect

        if self._debug:
            add_stream_logger()
            log.info("Creating MetaWearClient for {0}...".format(address))

        self.mw = MetaWear(self._address, hci_mac=device)

        log.debug("Client started for BLE device {0}...".format(self._address))

        self.accelerometer = None
        #self.gpio = None
        self.gyroscope = None
        self.magnetometer = None
        self.barometer = None
        self.ambient_light = None
        self.switch = None
        self.settings = None
        self.temperature = None
        self.haptic = None
        self.led = None
        self.sensorfusion = None

        if connect:
            self.connect()

    def __enter__(self):
        if not self._connect:
            self.connect()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        try:
            self.disconnect()
        except Exception as e:
            print("Could not disconnect: {0}".format(e))

    @property
    def board(self):
        return self.mw.board

    @property
    def firmware_version(self):
        return self.mw.info['firmware']

    @property
    def hardware_version(self):
        return self.mw.info['hardware']

    @property
    def manufacturer(self):
        return self.mw.info['manufacturer']

    @property
    def serial(self):
        return self.mw.info['serial']

    @property
    def model(self):
        return self.mw.info['model']

    def __str__(self):
        return "MetaWearClient, {0}: {1}".format(
            self._address, self.mw.info)

    def __repr__(self):
        return "<MetaWearClient, {0}>".format(self._address)

    def connect(self):
        """Connect this client to the MetaWear device."""
        self.mw.connect()
        self._initialize_modules()

    def disconnect(self):
        """Disconnects this client from the MetaWear device."""
        self.mw.disconnect()

    def _initialize_modules(self):
        #self.gpio = modules.GpioModule(
        #    self.board,
        #    libmetawear.mbl_mw_metawearboard_lookup_module(
        #        self.board, modules.Modules.MBL_MW_MODULE_GPIO))
        self.accelerometer = modules.AccelerometerModule(
            self.board,
            libmetawear.mbl_mw_metawearboard_lookup_module(
                self.board, modules.Modules.MBL_MW_MODULE_ACCELEROMETER))
        self.gyroscope = modules.GyroscopeModule(
            self.board,
            libmetawear.mbl_mw_metawearboard_lookup_module(
                self.board, modules.Modules.MBL_MW_MODULE_GYRO))
        self.magnetometer = modules.MagnetometerModule(
            self.board,
            libmetawear.mbl_mw_metawearboard_lookup_module(
                self.board, modules.Modules.MBL_MW_MODULE_MAGNETOMETER))
        self.barometer = modules.BarometerModule(
            self.board,
            libmetawear.mbl_mw_metawearboard_lookup_module(
                self.board, modules.Modules.MBL_MW_MODULE_BAROMETER))
        self.ambient_light = modules.AmbientLightModule(
            self.board,
            libmetawear.mbl_mw_metawearboard_lookup_module(
                self.board, modules.Modules.MBL_MW_MODULE_AMBIENT_LIGHT))
        self.switch = modules.SwitchModule(self.board)
        self.settings = modules.SettingsModule(self.board)
        self.temperature = modules.TemperatureModule(self.board)
        self.haptic = modules.HapticModule(self.board)
        self.led = modules.LEDModule(self.board)
        self.sensorfusion = modules.SensorFusionModule(
            self.board,
            libmetawear.mbl_mw_metawearboard_lookup_module(
                self.board, modules.Modules.MBL_MW_MODULE_SENSOR_FUSION))



# -*- coding: utf-8 -*-
"""
PyMetaWear
"""

import sys

from mbientlab.metawear import libmetawear

from pymetawear.version import __version__, version  # flake8: noqa

import logging
try:  # Python 2.7+
    from logging import NullHandler
except ImportError:
    class NullHandler(logging.Handler):
        def emit(self, record):
            pass

logging.getLogger(__name__).addHandler(NullHandler())


def add_stream_logger(stream=sys.stdout, level=logging.DEBUG):
    """
    Helper for quickly adding a StreamHandler to the logger. Useful for
    debugging.
    Returns the handler after adding it.
    """
    logger = logging.getLogger(__name__)
    has_stream_handler = any([isinstance(hndl, logging.StreamHandler) for hndl in logger.handlers])
    if has_stream_handler:
        return logger.handlers[-1]
    handler = logging.StreamHandler(stream=stream)
    handler.setFormatter(logging.Formatter('%(asctime)s %(levelname)s %(message)s'))
    logger.addHandler(handler)
    logger.setLevel(level)
    logger.debug('Added a {0} logging handler to logger: {1}'.format(stream.name, __name__))
    return handler



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:mod:`accelerometer`
==================

Updated by lkasso <hello@mbientlab.com>
Created by hbldh <henrik.blidh@nedomkull.com>
Created on 2016-04-10

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import time
from ctypes import c_ubyte, POINTER, cast, create_string_buffer

from pymetawear.discover import select_device
from pymetawear.client import MetaWearClient

address = select_device()
c = MetaWearClient(str(address), debug=True)
print("New client created: {0}".format(c))


#print("Set connection parameters...")
#c.settings.set_connection_parameters(min_conn_interval=750, max_conn_interval=1000, latency=128, timeout=16384)

#time.sleep(2.0)

print("Set new device name...")
c.settings.set_device_name(name="hbldh")

time.sleep(2.0)

print("Set Tx power to -20...")
c.settings.set_tx_power(power=-20)

time.sleep(2.0)

scan_response = "\x03\x03\xD8\xfe\x10\x16\xd8\xfe\x00\x12\x00\x6d\x62\x69\x65\x6e\x74\x6c\x61\x62\x00"
print("Set custom scan response...")
c.settings.set_scan_response(response=scan_response)

time.sleep(2.0)

print("Set advertising parameters...")
c.settings.set_ad_interval(interval=417, timeout=0)

time.sleep(2.0)

print("Start advertising...")
c.settings.start_advertising()

#this is a good time to call hcidump and verify changes
time.sleep(1.0)

c.disconnect()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:mod:`gyroscope`
==================

Created by hbldh <henrik.blidh@nedomkull.com>
Created on 2016-04-26

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import time

from pymetawear.discover import select_device
from pymetawear.client import MetaWearClient

address = select_device()
c = MetaWearClient(str(address), debug=True)
print("New client created: {0}".format(c))

print("Write gyroscope settings...")
settings = c.gyroscope.get_possible_settings()
print(settings)

print("Write gyroscope settings...")
c.gyroscope.set_settings(data_rate=50.0, data_range=1000.0)

print("Subscribing to gyroscope signal notifications...")
c.gyroscope.notifications(lambda data: print(data))

time.sleep(10.0)

print("Unsubscribe to notification...")
c.gyroscope.notifications(None)

time.sleep(5.0)

c.disconnect()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:mod:`sensorfusion`
==================

Created by mgeorgi <marcus.georgi@kinemic.de>
Created on 2016-02-01

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import time
from functools import partial

from pymetawear.discover import select_device
from pymetawear.client import MetaWearClient
from mbientlab.metawear.cbindings import SensorFusionData, SensorFusionGyroRange, SensorFusionAccRange, SensorFusionMode

address = select_device()
c = MetaWearClient(str(address), debug=True)
print("New client created: {0}".format(c))

def handle_notification(data, type_=''):
    print("{0}: {1}".format(type_, data))

def handle_notification2(data):
    print("2: {0}".format(data))

print("Write Sensor Fusion settings...")
c.sensorfusion.set_mode(SensorFusionMode.NDOF)
#c.sensorfusion.set_acc_range(SensorFusionAccRange._8G)
#c.sensorfusion.set_gyro_range(SensorFusionGyroRange._1000DPS)

print("Set Time Processor to limit data rate to 50Hz for each channel")
#c.sensorfusion.set_sample_delay(SensorFusionData.EULER_ANGLE, 20)
c.sensorfusion.set_sample_delay(SensorFusionData.QUATERNION, 20)
c.sensorfusion.set_sample_delay(SensorFusionData.CORRECTED_ACC, 20)
#c.sensorfusion.set_sample_delay(SensorFusionData.CORRECTED_GYRO, 20)

print("Subscribing to Sensor Fusion Quaternion signal notifications...")
#c.sensorfusion.notifications(euler_angle_callback=handle_notification)
#c.sensorfusion.notifications(quaternion_callback=handle_notification)
c.sensorfusion.notifications(
    corrected_acc_callback=partial(handle_notification, type_='Acc '),
    quaternion_callback=partial(handle_notification, type_='Quat'),
    corrected_gyro_callback=partial(handle_notification, type_='Gyro'))

#time.sleep(2.0)

#c.sensorfusion.notifications(quaternion_callback=handle_notification2)

time.sleep(10.0)

print("Unsubscribe to notification...")
c.sensorfusion.notifications()

time.sleep(5.0)

c.disconnect()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
GPIO module
-----------

Created by hbldh <henrik.blidh@nedomkull.com> on 2016-04-14
Modified by lkasso <hello@mbientlab.com>

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import re
import logging

from pymetawear import libmetawear

from mbientlab.metawear.cbindings import GpioPinChangeType, GpioPullMode, \
    GpioAnalogReadParameters, GpioAnalogReadMode
from pymetawear.modules.base import PyMetaWearModule, data_handler

log = logging.getLogger(__name__)


class GpioModule(PyMetaWearModule):
    """MetaWear gpio module implementation.

    :param ctypes.c_long board: The MetaWear board pointer value.
    :param int module_id: The module id of this accelerometer
        component, obtained from ``libmetawear``.
    :param bool debug: If ``True``, module prints out debug information.

    """

    def __init__(self, board, module_id):
        super(GpioModule, self).__init__(board)
        self.module_id = module_id
        
        self.current_pin = 0
        self.current_mode = 0
        self.current_read = 0

        self.pin = {}
        self.mode = {}
        self.read = {}
        
        gpio_pin_class = GpioPinChangeType
        gpio_mode_class = GpioPullMode
        gpio_read_class = GpioAnalogReadMode

        if gpio_pin_class is not None:
            # Parse possible pin outputs for GPIOs.
            for key, value in vars(gpio_pin_class).items():
                if re.search('^([A-Z]*)', key) and isinstance(value,int):
                    self.pin.update({key:value})
        
        if gpio_mode_class is not None:
            # Parse possible pull modes for GPIOs.
            for key, value in vars(gpio_mode_class).items():
                if re.search('^([A-Z]*)', key) and isinstance(value,int):
                    self.mode.update({key:value})
        
        if gpio_read_class is not None:
            # Parse possible analog read modes for GPIOs.
            for key, value in vars(gpio_read_class).items():
                if re.search('^([A-Z]*)', key) and isinstance(value,int):
                    self.read.update({key:value})

    def __str__(self):
        return "{0} {1}: Pin outputs: {2}, Pull modes: {3}, Read modes: {4}".format(
            self.module_name, self.sensor_name,
            [k for k in sorted(self.pin.keys())],
            [k for k in sorted(self.mode.keys())],
            [k for k in sorted(self.read.keys())])

    def __repr__(self):
        return str(self)

    @property
    def module_name(self):
        return "Gpio"

    @property
    def sensor_name(self):
        return self.module_name

    @property
    def data_signal(self):
        # TODO: Fix this pin issue!
        if self.analog:
            return libmetawear.mbl_mw_gpio_get_analog_input_data_signal(self.board, pin)
        elif self.digital:
            return libmetawear.mbl_mw_gpio_get_analog_input_data_signal(
                self.board, pin)
        else:
            return libmetawear.mbl_mw_gpio_get_pin_monitor_data_signal(
                self.board)

    def _get_pin(self, value):
        if value.lower() in self.pin:
            return self.pin.get(value.lower())
        else:
            raise ValueError(
                "Requested Pin Change Type ({0}) was not part of possible values: {1}".format(
                    value.lower(), self.pin.keys()))

    def _get_mode(self, value):
        if value.lower() in self.mode:
            return self.mode.get(value.lower())
        else:
            raise ValueError(
                "Requested Pull Mode ({0}) was not part of possible values: {1}".format(
                    value.lower(), self.mode.keys()))
    
    def _get_read(self, value):
        if value.lower() in self.read:
            return self.read.get(value.lower())
        else:
            raise ValueError(
                "Requested Analog Read Mode ({0}) was not part of possible values: {1}".format(
                    value.lower(), self.read.keys()))

    def get_current_settings(self):
        return "Pin change type: {} pull mode: {} analog read mode: {}".format(self.current_pin, self.current_mode, self.current_read)

    def get_possible_settings(self):
        return {
            'Pin change type': [x.lower() for x in sorted(
                self.pin.keys(), key=lambda x:(x.lower()))],
            'Pull mode': [x.lower() for x in sorted(
                self.mode.keys(), key=lambda x:(x.lower()))],
            'Analog read mode': [x.lower() for x in sorted(
                self.read.keys(), key=lambda x:(x.lower()))]
        }

    def set_settings(self, pin, rmode=None, pmode=None, ptype=None):
        if rmode is not None:
            rmode = self._get_read(rmode)
            log.debug("Setting the Analog Read Mode to {0}".format(rmode))
            self.current_mode = rmode

        if pmode is not None:
            self.current_ = pmode

        if ptype is not None:
            self.current_ = ptype

    def set_pull_mode(self, pin=None, pmode=None):
        libmetawear.mbl_mw_gpio_set_pull_mode(self.board, pin, pmode)
    
    def set_digital_out(self, pin=None):
        libmetawear.mbl_mw_gpio_set_digital_output(self.board, pin)

    def clear_digital_out(self, pin=None):
        libmetawear.mbl_mw_gpio_clear_digital_output(self.board, pin)
    
    def set_pin_change(self, pin=None, ptype=None):
        libmetawear.mbl_mw_gpio_set_pin_change_type(self.board, pin, ptype)

    def notifications(self, callback=None):
        """Subscribe or unsubscribe to accelerometer notifications.

        Convenience method for handling gpio usage.

        Example:

        .. code-block:: python

            def handle_gpio_notification(data)
                # Handle a (epoch_time, int value) type.
                epoch = data[0]
                value = data[1]
                print("[{0}] value: {1}".format(epoch, *value))

            mwclient.gpio.notifications(handle_gpio_notification)

        :param callable callback: Gpio notification callback function.
            If `None`, unsubscription to accelerometer notifications is registered.

        """

        if callback is None:
            self.stop()
            super(GpioModule, self).notifications(None)
        else:
            super(GpioModule, self).notifications(data_handler(callback))
            self.start()

    def start(self, pin=None):
        """Switches the gpio to active mode."""
        libmetawear.mbl_mw_gpio_start_pin_monitoring(self.board, pin)

    def stop(self, pin=None):
        """Switches the gpio to standby mode."""
        libmetawear.mbl_mw_gpio_stop_pin_monitoring(self.board, pin)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

__version__ = '0.12.0'
version = __version__  # backwards compatibility name



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Sensor Fusion module
--------------------

Created by mgeorgi <marcus.georgi@kinemic.de> on 2017-02-01

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import warnings
import logging
from threading import Event

from pymetawear import libmetawear
from pymetawear.exceptions import PyMetaWearException
from mbientlab.metawear.cbindings import SensorFusionAccRange, \
    SensorFusionData, SensorFusionGyroRange, SensorFusionMode, \
    SensorOrientation, FnVoid_VoidP_VoidP, FnVoid_VoidP_DataP, TimeMode
from pymetawear.modules.base import PyMetaWearLoggingModule, Modules, data_handler, context_callback

log = logging.getLogger(__name__)
PROCESSOR_SET_WAIT_TIME = 5


def require_fusion_module(f):
    def wrapper(*args, **kwargs):
        if getattr(args[0], 'available', False) is False:
            raise PyMetaWearException("There is no Sensor Fusion "
                                      "module on your MetaWear board!")
        return f(*args, **kwargs)

    return wrapper


class SensorFusionModule(PyMetaWearLoggingModule):
    """MetaWear accelerometer module implementation.

    :param ctypes.c_long board: The MetaWear board pointer value.
    :param int module_id: The module id of the sensorfusion
        component, obtained from ``libmetawear``.
    :param bool debug: If ``True``, module prints out debug information.

    """

    def __init__(self, board, module_id):
        super(SensorFusionModule, self).__init__(board)
        self.module_id = module_id

        if self.module_id == Modules.MBL_MW_MODULE_NA:
            # No sensor fusion present!
            self.available = False
        else:
            self.available = True

        self.board = board

        self.current_active_signal = None
        self._calibration_state_data_signal = None

        self._streams_to_enable = {
            SensorFusionData.CORRECTED_ACC: False,
            SensorFusionData.CORRECTED_GYRO: False,
            SensorFusionData.CORRECTED_MAG: False,
            SensorFusionData.QUATERNION: False,
            SensorFusionData.EULER_ANGLE: False,
            SensorFusionData.GRAVITY_VECTOR: False,
            SensorFusionData.LINEAR_ACC: False,
        }

        self._data_source_signals = {
            SensorFusionData.CORRECTED_ACC: None,
            SensorFusionData.CORRECTED_GYRO: None,
            SensorFusionData.CORRECTED_MAG: None,
            SensorFusionData.QUATERNION: None,
            SensorFusionData.EULER_ANGLE: None,
            SensorFusionData.GRAVITY_VECTOR: None,
            SensorFusionData.LINEAR_ACC: None,
        }

        self._callbacks = {}
        self._calibration_state_callback = None

    def __str__(self):
        return "{0}".format(self.module_name)

    def __repr__(self):
        return str(self)

    @require_fusion_module
    def set_sample_delay(self, data_source, delay=None, differential=False):
        """
        Change the delay between samples using the onboard time processor
        module to change the effective sampling rate of a specific data source.

        :param data_source: A data source from sensor.SensorFusion
        :param delay: The delay in ms between samples,
            or None to reset to default
        :param differential: Set Time Preprocessor mode to differential,
            instead of the default, absolute
        """
        if self._data_source_signals[data_source] is None:
            log.debug("Getting data signal for data source {0}".format(
                data_source
            ))
            self._data_source_signals[data_source] = \
                libmetawear.mbl_mw_sensor_fusion_get_data_signal(
                    self.board, data_source
                )

        if delay is not None:
            mode = TimeMode.DIFFERENTIAL if differential else \
                TimeMode.ABSOLUTE
            log.debug("Creating time dataprocessor for signal {0}".format(
                self._data_source_signals[data_source]
            ))
            _done = Event()

            def _processor_set(context, processor):
                """
                Set global variables as the libmetawear callback can't handle the self
                parameter of instance methods.

                :param context: Pointer to additional data for the callback function
                :param processor: The processor that was created

                """
                self._data_source_signals[data_source] = processor
                _done.set()

            processor_set_func = FnVoid_VoidP_VoidP(_processor_set)
            libmetawear.mbl_mw_dataprocessor_time_create(
                self._data_source_signals[data_source],
                mode,
                delay,
                None,
                processor_set_func)
            _done.wait(timeout=PROCESSOR_SET_WAIT_TIME)

            if self._data_source_signals[data_source] is None:
                raise PyMetaWearException("Can't set data processor!")
        else:
            data_signal = libmetawear.mbl_mw_sensor_fusion_get_data_signal(
                    self.board, data_source)
            if self._data_source_signals[data_source] != data_signal:
                libmetawear.mbl_mw_dataprocessor_remove(
                    self._data_source_signals[data_source]
                )
                self._data_source_signals[data_source] = data_signal

    def _delay_set(self, processor):
        self._current_processor = processor
        self._waiting_for_processor = False

    @require_fusion_module
    def set_mode(self, mode):
        libmetawear.mbl_mw_sensor_fusion_set_mode(self.board,
                                                  mode)
        libmetawear.mbl_mw_sensor_fusion_write_config(self.board)

    @require_fusion_module
    def set_acc_range(self, acc_range):
        libmetawear.mbl_mw_sensor_fusion_set_acc_range(self.board,
                                                       acc_range)
        libmetawear.mbl_mw_sensor_fusion_write_config(self.board)

    @require_fusion_module
    def set_gyro_range(self, gyro_range):
        libmetawear.mbl_mw_sensor_fusion_set_gyro_range(self.board,
                                                        gyro_range)
        libmetawear.mbl_mw_sensor_fusion_write_config(self.board)

    @require_fusion_module
    def get_data_signal(self, data_source):
        if self._data_source_signals[data_source] is None:
            self._data_source_signals[data_source] = \
                libmetawear.mbl_mw_sensor_fusion_get_data_signal(
                    self.board, data_source
                )
        return self._data_source_signals[data_source]

    @require_fusion_module
    def get_calibration_state_data_signal(self):
        if self._calibration_state_data_signal is None:
            self._calibration_state_data_signal = \
                libmetawear.mbl_mw_sensor_fusion_calibration_state_data_signal(
                    self.board)
        return self._calibration_state_data_signal

    @property
    def data_signal(self):
        return self.current_active_signal

    @property
    def calibration_state_data_signal(self):
        return self._calibration_state_data_signal

    @property
    def module_name(self):
        return "Sensor Fusion"

    def get_current_settings(self):
        raise NotImplementedError()

    def get_possible_settings(self):
        raise NotImplementedError()

    def set_settings(self):
        raise NotImplementedError()

    def read_calibration_state(self):
        """Triggers the calibration state notification.

        N.B. that a :meth:`~calibration_notifications` call that registers a
        callback for calibration state should have been done prior to calling
        this method.

        """
        if self._calibration_state_callback is None:
            warnings.warn("No calibration state callback is registered!",
                          RuntimeWarning)
        libmetawear.mbl_mw_datasignal_read(self.calibration_state_data_signal)

    @require_fusion_module
    def notifications(self,
                      corrected_acc_callback=None,
                      corrected_gyro_callback=None,
                      corrected_mag_callback=None,
                      quaternion_callback=None,
                      euler_angle_callback=None,
                      gravity_callback=None,
                      linear_acc_callback=None,
                      calibration_state_callback=None):
        """Subscribe or unsubscribe to sensor fusion notifications.

        Convenience method for handling sensor fusion usage.

        Example:

        .. code-block:: python

            def handle_notification(data):
                # Handle dictionary with [epoch, value] keys.
                epoch = data["epoch"]
                xyz = data["value"]
                print(str(data))

            mwclient.sensorfusion.notifications(
                corrected_acc_callback=handle_notification)

        :param callable corrected_acc_callback: Acceleration notification
            callback function.
            If `None`, unsubscription to acceleration notifications is
            registered.
        :param callable corrected_gyro_callback: Gyroscope notification
            callback function.
            If `None`, unsubscription to gyroscope notifications is registered.
        :param callable corrected_mag_callback: Magnetometer notification
            callback function.
            If `None`, unsubscription to magnetometer notifications is
            registered.
        :param callable quaternion_callback: Quaternion notification callback
            function.
            If `None`, unsubscription to quaternion notifications is registered.
        :param callable euler_angle_callback: Euler angle notification callback
            function.
            If `None`, unsubscription to euler angle notifications is
            registered.
        :param callable gravity_callback: Gravity vector notification callback
            function.
            If `None`, unsubscription to gravity notifications is registered.
        :param callable linear_acc_callback: Linear acceleration notification
            callback function.
            If `None`, unsubscription to linear acceleration notifications is
            registered.
        :param callable calibration_state_callback: Calibration state notification
            callback function.
            If `None`, unsubscription to calibration state notifications is
            registered.

        """
        callback_data_source_map = {
            SensorFusionData.CORRECTED_ACC: corrected_acc_callback,
            SensorFusionData.CORRECTED_GYRO: corrected_gyro_callback,
            SensorFusionData.CORRECTED_MAG: corrected_mag_callback,
            SensorFusionData.QUATERNION: quaternion_callback,
            SensorFusionData.EULER_ANGLE: euler_angle_callback,
            SensorFusionData.GRAVITY_VECTOR: gravity_callback,
            SensorFusionData.LINEAR_ACC: linear_acc_callback
        }

        for data_source in callback_data_source_map:
            if callback_data_source_map[data_source] is not None:
                self._streams_to_enable[data_source] = True
            else:
                self._streams_to_enable[data_source] = False

        enable = False in [x is None for x in callback_data_source_map.values()]
        log.debug("Enable: %s" % enable)

        self._calibration_state_callback = calibration_state_callback

        if not enable:
            self.stop()
            self.toggle_sampling(False)

        for data_source in callback_data_source_map:
            self.current_active_signal = self.get_data_signal(data_source)
            callback = callback_data_source_map[data_source]
            if callback is not None:
                self.check_and_change_callback(
                    self.get_data_signal(data_source),
                    data_handler(callback)
                )
            else:
                self.check_and_change_callback(
                    self.get_data_signal(data_source),
                    None
                )

        if calibration_state_callback is not None:
            self.check_and_change_callback(
                self.get_calibration_state_data_signal(),
                data_handler(calibration_state_callback))
        else:
            self.check_and_change_callback(
                self.get_calibration_state_data_signal(),
                None)

        if enable:
            self.toggle_sampling(True)
            self.start()

    def check_and_change_callback(self, data_signal, callback):
        if callback is not None:

            log.debug("Subscribing to {0} changes. (Sig#: {1})".format(
                self.module_name, data_signal))
            if data_signal in self._callbacks:
                log.debug('Replacing callback for datasignal {0}...'.format(
                    data_signal))
                libmetawear.mbl_mw_datasignal_unsubscribe(data_signal)
                self._callbacks.pop(data_signal)
            callback = context_callback(callback)
            self._callbacks[data_signal] = (callback, FnVoid_VoidP_DataP(callback))
            libmetawear.mbl_mw_datasignal_subscribe(
                data_signal, None, self._callbacks[data_signal][1])
        else:
            if data_signal not in self._callbacks:
                return
            log.debug("Unsubscribing to {0} changes. (Sig#: {1})".format(
                self.module_name, data_signal))
            libmetawear.mbl_mw_datasignal_unsubscribe(data_signal)
            self._callbacks.pop(data_signal)

    @require_fusion_module
    def start(self):
        """Switches the sensorfusion to active mode."""
        libmetawear.mbl_mw_sensor_fusion_start(self.board)

    @require_fusion_module
    def stop(self):
        """Switches the sensorfusion to standby mode."""
        libmetawear.mbl_mw_sensor_fusion_stop(self.board)

    @require_fusion_module
    def toggle_sampling(self, enabled=True):
        """Enables or disables sensor fusion sampling.

        :param bool enabled: Desired state of the sensor fusion module.

        """
        if enabled:
            for data_source in self._streams_to_enable:
                if self._streams_to_enable[data_source]:
                    libmetawear.mbl_mw_sensor_fusion_enable_data(
                        self.board, data_source)
        else:
            libmetawear.mbl_mw_sensor_fusion_clear_enabled_mask(
                self.board)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Switch module
-------------

Created by hbldh <henrik.blidh@nedomkull.com> on 2016-04-14

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import logging

from pymetawear import libmetawear
from pymetawear.modules.base import PyMetaWearModule, data_handler

log = logging.getLogger(__name__)


class SwitchModule(PyMetaWearModule):
    """MetaWear Switch module implementation.

    :param ctypes.c_long board: The MetaWear board pointer value.
    :param bool debug: If ``True``, module prints out debug information.

    """

    def __str__(self):
        return "{0}".format(
            self.module_name)

    def __repr__(self):
        return str(self)

    @property
    def module_name(self):
        return "Switch"

    @property
    def data_signal(self):
        return libmetawear.mbl_mw_switch_get_state_data_signal(self.board)

    def notifications(self, callback=None):
        """Subscribe or unsubscribe to switch notifications.

        Convenience method for handling switch usage.

        Example:

        .. code-block:: python

            def switch_callback(data):
                # Handle dictionary with [epoch, value] keys.
                epoch = data["epoch"]
                value = data["value"]
                if value == 1:
                    print("[{0}] Switch pressed!".format(epoch))
                elif status == 0:
                    print("[{0}] Switch released!".format(epoch))

            mwclient.switch.notifications(switch_callback)

        :param callable callback: Switch notification callback function.
            If `None`, unsubscription to switch notifications is registered.

        """
        super(SwitchModule, self).notifications(
            data_handler(callback) if callback is not None else None)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Base module
-----------

Created by hbldh <henrik.blidh@nedomkull.com> on 2016-04-14
Modified by lkasso <hello@mbientlab.com>

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import logging
from copy import deepcopy
from ctypes import c_int, c_uint, c_float, cast, POINTER, c_ubyte, byref
from functools import wraps
from threading import Event

import tqdm

from pymetawear import libmetawear
from pymetawear.exceptions import PyMetaWearException, PyMetaWearDownloadTimeout
from mbientlab.metawear.cbindings import FnVoid_VoidP_DataP,  \
    DataTypeId, CartesianFloat, BatteryState, Tcs34725ColorAdc, EulerAngles, \
    CalibrationState, Quaternion, CorrectedCartesianFloat, FnVoid_VoidP_VoidP, \
    LogDownloadHandler, FnVoid_VoidP_UInt_UInt, \
    FnVoid_VoidP_UByte_Long_UByteP_UByte

log = logging.getLogger(__name__)


class Modules(object):
    """Class for storing PyMetaWear module identifiers."""

    MBL_MW_MODULE_NA = -1
    MBL_MW_MODULE_SWITCH = 1
    MBL_MW_MODULE_LED = 2
    MBL_MW_MODULE_ACCELEROMETER = 3
    MBL_MW_MODULE_TEMPERATURE = 4
    MBL_MW_MODULE_GPIO = 5
    MBL_MW_MODULE_NEO_PIXEL = 6
    MBL_MW_MODULE_IBEACON = 7
    MBL_MW_MODULE_HAPTIC = 8
    MBL_MW_MODULE_DATA_PROCESSOR = 9
    MBL_MW_MODULE_EVENT = 0xa
    MBL_MW_MODULE_LOGGING = 0xb
    MBL_MW_MODULE_TIMER = 0xc
    MBL_MW_MODULE_I2C = 0xd
    MBL_MW_MODULE_MACRO = 0xf
    MBL_MW_MODULE_GSR = 0x10
    MBL_MW_MODULE_SETTINGS = 0x11
    MBL_MW_MODULE_BAROMETER = 0x12
    MBL_MW_MODULE_GYRO = 0x13
    MBL_MW_MODULE_AMBIENT_LIGHT = 0x14
    MBL_MW_MODULE_MAGNETOMETER = 0x15
    MBL_MW_MODULE_HUMIDITY = 0x16
    MBL_MW_MODULE_COLOR_DETECTOR = 0x17
    MBL_MW_MODULE_PROXIMITY = 0x18
    MBL_MW_MODULE_SENSOR_FUSION = 0x19
    MBL_MW_MODULE_DEBUG = 0xfe


class PyMetaWearModule(object):
    """Base class for PyMetaWear module implementations.

    :param ctypes.c_long board: The MetaWear board pointer value.
    :param bool debug: If ``True``, module prints out debug information.

    """

    def __init__(self, board):
        self.board = board
        self.is_present = True
        self.callback = None

    def __str__(self):
        return "PyMetaWearModule"

    def __repr__(self):
        return super(PyMetaWearModule, self).__repr__()

    @property
    def module_name(self):
        """Get module name.

        :return: The name of this module.
        :rtype: str

        """
        return ''

    @property
    def sensor_name(self):
        """Get sensor name, if applicable.

        :return: The name of this module.
        :rtype: str or None

        """
        return None

    @property
    def data_signal(self):
        """Returns the data signal pointer value for the switch module.

        :returns: The pointer value. (Long if on x64 architecture.)
        :rtype: :py:class:`ctypes.c_long` or :py:class:`ctypes.c_int`

        """
        raise PyMetaWearException(
            "No data signal exists for {0} module.".format(self))

    def set_settings(self, **kwargs):
        raise PyMetaWearException(
            "No settings exists for {0} module.".format(self))

    def get_current_settings(self):
        return {}

    def get_possible_settings(self):
        return {}

    def notifications(self, callback=None):
        """Toggle notifications/subscriptions to data signals
        on the MetaWear board.

        :param callable callback: The function to call when
            data signal notification arrives. If ``None``, an
            unsubscription to notifications is sent.

        """
        data_signal = self.data_signal
        if callback is not None:
            log.debug("Subscribing to {0} changes. (Sig#: {1})".format(
                self.module_name, data_signal))
            if self.callback is not None:
                raise PyMetaWearException(
                    "Subscription to {0} signal already in place!")
            # Handle the context argument added in MetaWear C++ API 0.12.
            callback = context_callback(callback)
            self.callback = (callback, FnVoid_VoidP_DataP(callback))
            libmetawear.mbl_mw_datasignal_subscribe(
                data_signal, None, self.callback[1])
        else:
            log.debug("Unsubscribing to {0} changes. (Sig#: {1})".format(
                self.module_name, data_signal))
            if self.callback is None:
                return
            libmetawear.mbl_mw_datasignal_unsubscribe(data_signal)
            self.callback = None


class PyMetaWearLoggingModule(PyMetaWearModule):
    """Special class with additions for modules with logging support."""

    def __init__(self, board):
        super(PyMetaWearLoggingModule, self).__init__(board)

        self._logger_ready_event = None
        self._data_received = None
        self._download_done = False
        self._logger_running = False
        self._logger_address = None

        self._progress_bar = None

        self._logged_data = []

    def _logger_ready(self, address):
        if address:
            self._logger_address = address
            log.debug("Logger address: {0}".format(self._logger_address))
        else:
            # Do nothing here. Let main thread handle lack of address.
            pass
        self._logger_ready_event.set()

    def _progress_update(self, entries_left, total_entries):
        if self._progress_bar is None:
            self._progress_bar = tqdm.tqdm(total=total_entries)
        n_read_entries = total_entries - entries_left
        self._progress_bar.update(n_read_entries - self._progress_bar.n)
        self._data_received.set()
        if entries_left == 0:
            self._progress_bar.update(
                total_entries - self._progress_bar.n)
            self._progress_bar.close()
            self._progress_bar = None
            self._download_done = True

    def _unknown_entry(self, id, epoch, data, length):
        """Handle unknown data entries in the log.

        I have no idea what this data is. Needs further investigation.

        :param id (int):
        :param epoch (int):
        :param data:
        :param length (int):

        """
        self._data_received.set()
        log.debug('Unknown Entry: ID: {0}, epoch: {1}, '
                  'data: {2}, Length: {3}'.format(
            id, epoch, bytearray(data)[:length], length))

    def _unhandled_entry(self, data):
        self._data_received.set()
        log.debug('Unhandled Entry: ' + str(data))

    def _default_download_callback(self, data):
        self._logged_data.append(data)

    def start(self):
        raise NotImplementedError("Must be implemented by module.")

    def stop(self):
        raise NotImplementedError("Must be implemented by module.")

    def toggle_sampling(self, enabled=True):
        raise NotImplementedError("Must be implemented by module.")

    def start_logging(self):
        """Setup and start logging of data signals on the MetaWear board"""
        data_signal = self.data_signal
        if getattr(self, 'high_frequency_stream', False):
            raise PyMetaWearException(
                "Cannot log on high frequency stream signal.")
        self._logger_ready_event = Event()
        logger_ready = FnVoid_VoidP_VoidP(context_callback(self._logger_ready))
        libmetawear.mbl_mw_datasignal_log(self.data_signal, None, logger_ready)
        self._logger_ready_event.wait()
        if self._logger_address is None:
            raise PyMetaWearException(
                'Failed to start logging for {0} module!'.format(
                    self.module_name))

        log.debug("Start Logger (Logger#: {0}, Signal#: {1})".format(
            self._logger_address, data_signal))

        self._logger_running = True
        self._download_done = False
        libmetawear.mbl_mw_logging_start(self.board, 0)
        self.toggle_sampling(True)
        self.start()

    def stop_logging(self):
        """Stop logging of data signals on the MetaWear board"""
        self.stop()
        self.toggle_sampling(False)
        log.debug("Stop Logger (Logger#: {0})".format(self._logger_address))

        libmetawear.mbl_mw_logging_stop(self.board)
        self._logger_running = False

    def download_log(
            self,
            timeout=3.0,
            data_callback=None,
            progress_update_function=None,
            unknown_entry_function=None,
            unhandled_entry_function=None
    ):
        """Download logged data from the MetaWear board

        :param timeout: Time to wait for download to resume if connection is lost.
        :param data_callback: Function called to process each downloaded sample.
        :param progress_update_function: Function called each
         `progress_update_each_n_sample` to give feedback on download progress.
        :param unknown_entry_function: Function called when unknown logging
         entries are encountered.
        :param unhandled_entry_function: Function called when unhandled entries
         are encountered.
        :return: The logged data, in case download was successful.

        """
        if self._logger_running:
            # Stop logging if it is active.
            self.stop_logging()

        if data_callback is None:
            data_callback = data_handler(self._default_download_callback)
        if progress_update_function is None:
            progress_update_function = self._progress_update
        if unknown_entry_function is None:
            unknown_entry_function = self._unknown_entry
        if unhandled_entry_function is None:
            unhandled_entry_function = self._unhandled_entry

        data_point_handler = FnVoid_VoidP_DataP(context_callback(data_callback))
        progress_update = FnVoid_VoidP_UInt_UInt(context_callback(progress_update_function))
        unknown_entry = FnVoid_VoidP_UByte_Long_UByteP_UByte(context_callback(unknown_entry_function))
        unhandled_entry = FnVoid_VoidP_DataP(data_handler(context_callback(unhandled_entry_function)))
        log_download_handler = LogDownloadHandler(
            received_progress_update=progress_update,
            received_unknown_entry=unknown_entry,
            received_unhandled_entry=unhandled_entry
        )

        log.debug("Subscribe to Logger. (Logger#: {0})".format(
            self._logger_address))
        libmetawear.mbl_mw_logger_subscribe(
            self._logger_address, None, data_point_handler)
        self._data_received = Event()
        log.debug("Waiting for completed download. (Logger#: {0})".format(
            self._logger_address))
        libmetawear.mbl_mw_logging_download(
            self.board, 1000,
            byref(log_download_handler))

        while not self._download_done:
            status = self._data_received.wait(timeout)
            self._data_received.clear()
            if not self._download_done and not status:
                if self._progress_bar is not None:
                    self._progress_bar.close()
                    self._progress_bar = None
                raise PyMetaWearDownloadTimeout(
                    "Bluetooth connection lost! Please reconnect and retry download...")

        log.debug("Download done. (Logger#: {0})".format(
            self._logger_address))
        log.debug("Remove logger. (Logger#: {0})".format(
            self._logger_address))
        libmetawear.mbl_mw_logger_remove(self._logger_address)

        logged_data = self._logged_data
        self._logged_data = []
        self._data_received = None

        return logged_data


def _error_handler(data):
    raise RuntimeError('Unrecognized data type id: ' +
                       str(data.contents.type_id))


def _byte_array_handler(data):
    ptr = cast(data.contents.value, POINTER(c_ubyte * data.contents.length))
    return [ptr.contents[i].value for i in range(0, data.contents.length)]


DATA_HANDLERS = {
    DataTypeId.UINT32: lambda x: cast(
        x.contents.value, POINTER(c_uint)).contents.value,
    DataTypeId.INT32: lambda x: cast(
        x.contents.value, POINTER(c_int)).contents.value,
    DataTypeId.FLOAT: lambda x: cast(
        x.contents.value, POINTER(c_float)).contents.value,
    DataTypeId.CARTESIAN_FLOAT: lambda x: cast(
        x.contents.value, POINTER(CartesianFloat)).contents,
    DataTypeId.BATTERY_STATE: lambda x: cast(
        x.contents.value, POINTER(BatteryState)).contents,
    DataTypeId.BYTE_ARRAY: _byte_array_handler,
    DataTypeId.TCS34725_ADC: lambda x: cast(
        x.contents.value, POINTER(Tcs34725ColorAdc)).contents,
    DataTypeId.EULER_ANGLE: lambda x: cast(
        x.contents.value, POINTER(EulerAngles)).contents,
    DataTypeId.QUATERNION: lambda x: cast(
        x.contents.value, POINTER(Quaternion)).contents,
    DataTypeId.CORRECTED_CARTESIAN_FLOAT: lambda x: cast(
        x.contents.value, POINTER(CorrectedCartesianFloat)).contents,
    DataTypeId.CALIBRATION_STATE: lambda x: cast(
        x.contents.value, POINTER(CalibrationState)).contents,
}


def context_callback(func):
    """Decorator to handle the context argument added in MetaWear C++ API 0.12

    This method is used internally so that the end-user should not have to
    know about and handle the ``context`` argument.

    :param func: The function to add context argument to.
    :return: The wrapped function.

    """
    @wraps(func)
    def wrapper(context, *args):
        func(*args)
    return wrapper


def data_handler(func):
    @wraps(func)
    def wrapper(data):
        func({
            'epoch': int(data.contents.epoch),
            'value': deepcopy(DATA_HANDLERS.get(
                data.contents.type_id, _error_handler)(data)),
        })

    return wrapper



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:mod:'accelerometer_logging.py'
==================
Updated by dmatthes1982 <dmatthes@cbs.mpg.de>
Created by hbldh <henrik.blidh@nedomkull.com>
Created on 2018-04-20
"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import time

from pymetawear.discover import select_device
from pymetawear.client import MetaWearClient
from pymetawear.exceptions import PyMetaWearException, PyMetaWearDownloadTimeout

address = select_device()

client = MetaWearClient(str(address), debug=False)
print("New client created: {0}".format(client))

settings = client.accelerometer.get_possible_settings()
print("Possible accelerometer settings of client:")
for k, v in settings.items():
    print(k, v)

print("Write accelerometer settings...")
client.accelerometer.set_settings(data_rate=400, data_range=4.0)

settings = client.accelerometer.get_current_settings()
print("Accelerometer settings of client: {0}".format(settings))

client.accelerometer.high_frequency_stream = False
client.accelerometer.start_logging()
print("Logging accelerometer data...")

time.sleep(10.0)

client.accelerometer.stop_logging()
print("Logging stopped.")

print("Downloading data...")
download_done = False
n = 0
data = None
while (not download_done) and n < 3:
    try:
        data = client.accelerometer.download_log()
        download_done = True
    except PyMetaWearDownloadTimeout:
        print("Download of log interrupted. Trying to reconnect...")
        client.disconnect()
        client.connect()
        n += 1
if data is None:
    raise PyMetaWearException("Download of logging data failed.")

for d in data:
    print(d)

print("Disconnecting...")
client.disconnect()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Magnetometer module
-------------------

Created by jboeer <jonas.boeer@kinemic.de> on 2016-09-07

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import re
import logging

from pymetawear import libmetawear
from pymetawear.exceptions import PyMetaWearException
from mbientlab.metawear.cbindings import MagBmm150Odr, MagBmm150Preset
from pymetawear.modules.base import PyMetaWearLoggingModule, Modules, data_handler

log = logging.getLogger(__name__)


def require_bmm150(f):
    def wrapper(*args, **kwargs):
        if getattr(args[0], 'mag_p_class', None) is None:
            raise PyMetaWearException("There is not Magnetometer "
                                      "module on your MetaWear board!")
        return f(*args, **kwargs)

    return wrapper


class MagnetometerModule(PyMetaWearLoggingModule):
    """MetaWear accelerometer module implementation.

    :param ctypes.c_long board: The MetaWear board pointer value.
    :param int module_id: The module id of this accelerometer
        component, obtained from ``libmetawear``.
    :param bool debug: If ``True``, module prints out debug information.

    """

    def __init__(self, board, module_id):
        super(MagnetometerModule, self).__init__(board)
        self.current_power_preset = None

        self.module_id = module_id
        
        self.power_presets = {}
        self.odr = {}

        if self.module_id == Modules.MBL_MW_MODULE_NA:
            # No magnetometer present!
            self.mag_o_class = None
            self.mag_p_class = None
        else:
            self.mag_o_class = MagBmm150Odr
            self.mag_p_class = MagBmm150Preset

        if self.mag_p_class is not None:
            # Parse possible presets for this magnetometer.
            for key, value in vars(self.mag_p_class).items():
                if re.search('^([A-Z\_]*)', key) and isinstance(value,int):
                    self.power_presets.update({key.lower(): value})

        if self.mag_o_class is not None:
            # Parse possible data rates for this magnetometer.
            for key, value in vars(self.mag_o_class).items():
                if re.search('^_([0-9\_]*Hz)', key) and isinstance(value,int):
                    self.odr.update({key[1:-2]: value})

    def __str__(self):
        return "{0} {1}: Power presets: {2}".format(
            self.module_name, self.sensor_name,
            [float(k) for k in sorted(self.power_presets.keys())])

    def __repr__(self):
        return str(self)

    @property
    def module_name(self):
        return "Magnetometer"

    @property
    def sensor_name(self):
        if self.mag_p_class is not None:
            return self.mag_p_class.__name__.replace(
                'Mag', '').replace('Preset', '')
        else:
            return ''

    @property
    @require_bmm150
    def data_signal(self):
        return libmetawear.mbl_mw_mag_bmm150_get_b_field_data_signal(self.board)

    def _get_power_preset(self, value):
        if value.lower() in self.power_presets:
            return self.power_presets.get(value.lower())
        else:
            raise ValueError("Requested power preset ({0}) was not part of "
                             "possible values: {1}".format(value.lower(), [self.power_presets.keys()]))

    @require_bmm150
    def get_current_settings(self):
        return {
            'power_preset': self.current_power_preset
        }

    @require_bmm150
    def get_possible_settings(self):
        return {
            'power_preset': [sorted(self.power_presets.keys(), key=lambda x:(x.lower()))],
        }

    @require_bmm150
    def set_settings(self, power_preset=None):
        """Set magnetometer settings.

         .code-block:: python

            mwclient.magnetometer.set_settings(power_preset="LOW_POWER")

        Call :meth:`~get_possible_settings` to see which values
        that can be set for this sensor.

        :param str power_preset: The power preset, influencing the data rate,
            accuracy and power consumption

        """
        if power_preset is not None:
            pp = self._get_power_preset(power_preset)
            log.debug("Setting Magnetometer power preset to {0}".format(pp))
            libmetawear.mbl_mw_mag_bmm150_set_preset(self.board, pp)
            self.current_power_preset = pp

    @require_bmm150
    def notifications(self, callback=None):
        """Subscribe or unsubscribe to magnetometer notifications.

        Convenience method for handling magnetometer usage.

        Example:

        .. code-block:: python

            def handle_notification(data):
                # Handle dictionary with [epoch, value] keys.
                epoch = data["epoch"]
                xyz = data["value"]
                print(str(data))

            mwclient.magnetometer.notifications(handle_notification)

        :param callable callback: Magnetometer notification callback function.
            If `None`, unsubscription to magnetometer notifications is registered.

        """
        if callback is None:
            self.stop()
            self.toggle_sampling(False)
            super(MagnetometerModule, self).notifications(None)
        else:
            super(MagnetometerModule, self).notifications(data_handler(callback))
            self.toggle_sampling(True)
            self.start()

    @require_bmm150
    def start(self):
        """Switches the magnetometer to active mode."""
        libmetawear.mbl_mw_mag_bmm150_start(self.board)

    @require_bmm150
    def stop(self):
        """Switches the magnetometer to standby mode."""
        libmetawear.mbl_mw_mag_bmm150_stop(self.board)

    @require_bmm150
    def toggle_sampling(self, enabled=True):
        """Enables or disables magnetometer sampling.

        :param bool enabled: Desired state of the magnetometer.

        """
        if enabled:
            libmetawear.mbl_mw_mag_bmm150_enable_b_field_sampling(self.board)
        else:
            libmetawear.mbl_mw_mag_bmm150_disable_b_field_sampling(self.board)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:mod:'magnetometer_logging.py'
==================
Created by hbldh <henrik.blidh@nedomkull.com>
Created on 2018-04-20

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import time

from pymetawear.discover import select_device
from pymetawear.client import MetaWearClient

address = select_device()

client = MetaWearClient(str(address), debug=False)
print("New client created: {0}".format(client))

settings = client.magnetometer.get_possible_settings()
print("Possible magnetometer settings of client:")
for k, v in settings.items():
    print(k, v)

print("Write magnetometer settings...")
client.magnetometer.set_settings(power_preset='REGULAR', odr='30')

# Not implemented yet...
# settings = client.magnetometer.get_current_settings()
# print("Magnetometer settings of client: {0}".format(settings))

time.sleep(0.2)

client.magnetometer.high_frequency_stream = False
client.magnetometer.start_logging()
print("Logging magnetometer data...")

time.sleep(5.0)

client.magnetometer.stop_logging()
print("Logging stopped.")

print("Downloading data...")
data = client.magnetometer.download_log()
for d in data:
    print(d)

print("Disconnecting...")
client.disconnect()
